@@ -89,9 +89,26 @@
 //! Additionally there will be a function `into_iron_response` which converts anything into
 //! a `hyper::Response` that can be converted into a `HttpApiProblem`.
 //!
+//! ### with_rocket(nightly only)
 //!
-//!  ## Recent changes
+//! There is a conversion between `rocket`s Status and `HttpStatusCode` back and forth.
 //!
+//! `HttpApiProblem` implements `rocket::response::Responder`, allowing it to be returned
+//! from rocket handlers directly (e.g. as `Result<T, HttpApiProblem>`).
+//! It also provides a method `to_rocket_response` which explicitly constructs a rocket `Response`.
+//! If the `status` field of the `HttpApiProblem` is `None` `500 - Internal Server Error` is the default.
+//!
+//! `From<HttpApiProblem` for `rocket::Response` will also be there. It simply calls
+//! `to_rocket_response`.
+//!
+//! Additionally there will be a function `into_rocket_response` which converts anything into
+//! a `rocket::Response` that can be converted into a `HttpApiProblem`.
+//!
+//!
+//! ## Recent changes
+//!
+//!  * 0.5.1
+//!      * Support for `Rocket` (contributed by panicbit)
 //!  * 0.5.0
 //!      * Breaking changes, features renamed to `with_iron` and `with_hyper`
 //!      * `to_iron_response` now takes a ref insted of `Self`.
@@ -115,6 +132,9 @@ extern crate hyper;
 #[cfg(feature = "with_hyper")]
 extern crate mime;
 
+#[cfg(feature = "with_rocket")]
+extern crate rocket;
+
 use std::fmt;
 
 use serde::{Deserialize, Deserializer, Serialize, Serializer};
@@ -353,10 +373,12 @@ impl HttpApiProblem {
         s
     }
 
+    /// Serialize to a JSON `Vec<u8>`
     pub fn json_bytes(&self) -> Vec<u8> {
         serde_json::to_vec(self).unwrap()
     }
 
+    /// Serialize to a JSON `String`
     pub fn json_string(&self) -> String {
         serde_json::to_string(self).unwrap()
     }
@@ -387,8 +409,8 @@ impl HttpApiProblem {
     /// default.
     #[cfg(feature = "with_hyper")]
     pub fn to_hyper_response(&self) -> hyper::Response {
-        use hyper::header::{ContentLength, ContentType};
         use hyper::StatusCode;
+        use hyper::header::{ContentLength, ContentType};
         use hyper::*;
         use mime::*;
 
@@ -405,6 +427,30 @@ impl HttpApiProblem {
 
         response
     }
+
+    /// Creates a `rocket` response.
+    ///
+    /// If status is `None` `500 - Internal Server Error` is the
+    /// default.
+    #[cfg(feature = "with_rocket")]
+    pub fn to_rocket_response(&self) -> rocket::Response<'static> {
+        use rocket::Response;
+        use rocket::http::ContentType;
+        use rocket::http::Status;
+        use std::io::Cursor;
+
+        let status: Status = self.status.unwrap_or(HttpStatusCode::InternalServerError).into();
+
+        let content_type: ContentType = PROBLEM_JSON_MEDIA_TYPE.parse().unwrap();
+        let json = self.json_bytes();
+        let response = Response::build()
+            .status(status)
+            .sized_body(Cursor::new(json))
+            .header(content_type)
+            .finalize();
+
+        response
+    }
 }
 
 impl From<HttpStatusCode> for HttpApiProblem {
@@ -430,6 +476,10 @@ impl From<HttpApiProblem> for ::iron::response::Response {
     }
 }
 
+/// Creates an `hyper::Response` from something that can become an `HttpApiProblem`.
+///
+/// If status is `None` `500 - Internal Server Error` is the
+/// default.
 #[cfg(feature = "with_hyper")]
 pub fn into_hyper_response<T: Into<HttpApiProblem>>(what: T) -> hyper::Response {
     let problem: HttpApiProblem = what.into();
@@ -443,6 +493,30 @@ impl From<HttpApiProblem> for hyper::Response {
     }
 }
 
+/// Creates an `rocket::Response` from something that can become an `HttpApiProblem`.
+///
+/// If status is `None` `500 - Internal Server Error` is the
+/// default.
+#[cfg(feature = "with_rocket")]
+pub fn into_rocket_response<T: Into<HttpApiProblem>>(what: T) -> ::rocket::Response<'static> {
+    let problem: HttpApiProblem = what.into();
+    problem.to_rocket_response()
+}
+
+#[cfg(feature = "with_rocket")]
+impl From<HttpApiProblem> for ::rocket::Response<'static> {
+    fn from(problem: HttpApiProblem) -> ::rocket::Response<'static> {
+        problem.to_rocket_response()
+    }
+}
+
+#[cfg(feature = "with_rocket")]
+impl<'r> ::rocket::response::Responder<'r> for HttpApiProblem {
+    fn respond_to(self, _request: &::rocket::Request) -> Result<::rocket::Response<'r>, ::rocket::http::Status> {
+        Ok(self.into())
+    }
+}
+
 /// An HTTP status code (`status-code` in RFC 7230 et al.).
 ///
 /// This enum contains all common status codes and an Unregistered
@@ -806,3 +880,20 @@ impl From<HttpStatusCode> for ::iron::status::Status {
         ::iron::status::Status::from_u16(status.to_u16())
     }
 }
+
+#[cfg(feature = "with_rocket")]
+impl From<::rocket::http::Status> for HttpStatusCode {
+    fn from(rocket_status: ::rocket::http::Status) -> HttpStatusCode {
+        rocket_status.code.into()
+    }
+}
+
+#[cfg(feature = "with_rocket")]
+impl From<HttpStatusCode> for ::rocket::http::Status {
+    fn from(status: HttpStatusCode) -> ::rocket::http::Status {
+        use rocket::http::Status;
+
+        let code = status.to_u16();
+        Status::from_code(code).unwrap_or(Status::new(code, ""))
+    }
+}
